from django import forms
from django.utils import timezone
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from ckeditor.fields import RichTextFormField

from tinyforum import signals
from tinyforum.models import Thread, Post, PostReport


class BaseForm(forms.Form):
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop("request")
        super().__init__(*args, **kwargs)

    def save(self, commit=True):
        instance = super().save(commit=False)
        if not instance.authored_by_id:
            instance.authored_by = self.request.user
        if commit:
            instance.save()
        return instance


class CreateThreadForm(BaseForm, forms.ModelForm):
    text = RichTextFormField(
        label=_("text"),
        config_name="tinyforum-post",
        extra_plugins=["emojione"],
        external_plugin_resources=[
            ("emojione", "/static/webapp/lib/ckeditor-emojione-1.0.1/", "plugin.js")
        ],
    )

    class Meta:
        model = Thread
        fields = ("title",)

    def save(self):
        instance = super().save()
        instance.starred_by.add(self.request.user)
        instance.posts.create(
            authored_by=instance.authored_by, text=self.cleaned_data["text"]
        )
        return instance


class UpdateThreadForm(BaseForm, forms.ModelForm):
    class Meta:
        model = Thread
        fields = ("title",)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.instance.closed_at:
            self.fields["close_thread"] = forms.BooleanField(
                label=_("Close thread"),
                required=False,
                help_text=_(
                    "The thread will still be visible, but no posts"
                    " can be added to it."
                ),
            )

    def save(self):
        instance = super().save(commit=False)
        if self.cleaned_data.get("close_thread"):
            instance.closed_at = timezone.now()
        instance.save()
        return instance


class ModerateThreadForm(UpdateThreadForm, forms.ModelForm):
    moderation_status = forms.ChoiceField(
        label=capfirst(_("moderation status")),
        choices=PostReport.MODERATION_STATUS_CHOICES,
        widget=forms.RadioSelect,
    )

    class Meta:
        model = Thread
        fields = ("title", "is_pinned", "moderation_status")


def form_for_thread(request, *, instance=None, moderation=False):
    kw = {
        "data": request.POST if request.method == "POST" else None,
        "request": request,
        "instance": instance,
    }
    if instance is None:
        return CreateThreadForm(**kw)
    elif moderation:
        return ModerateThreadForm(**kw)
    elif instance and request.user == instance.authored_by:
        return UpdateThreadForm(**kw)
    return None


class BasePostForm(BaseForm):
    def __init__(self, *args, **kwargs):
        self.thread = kwargs.pop("thread")
        super().__init__(*args, **kwargs)

    def save(self, commit=True):
        instance = super().save(commit=False)
        if not instance.thread_id:
            instance.thread = self.thread
        if commit:
            instance.save()
        return instance


class CreatePostForm(BasePostForm, forms.ModelForm):
    class Meta:
        model = Post
        fields = ("text",)

    def save(self):
        instance = super().save()
        signals.post_created.send(
            sender=Post, instance=instance, form=self, request=self.request
        )
        return instance


class UpdatePostForm(BasePostForm, forms.ModelForm):
    class Meta:
        model = Post
        fields = ("text",)


class ModeratePostForm(BasePostForm, forms.ModelForm):
    moderation_status = forms.ChoiceField(
        label=capfirst(_("moderation status")),
        choices=PostReport.MODERATION_STATUS_CHOICES,
        widget=forms.RadioSelect,
    )

    class Meta:
        model = Post
        fields = ("text", "moderation_status")


def form_for_post(request, *, thread, instance=None, moderation=False):
    kw = {
        "data": request.POST if request.method == "POST" else None,
        "request": request,
        "instance": instance,
        "thread": thread,
    }
    if thread.closed_at:
        return None
    elif moderation:
        return ModeratePostForm(**kw)
    elif instance and request.user == instance.authored_by:
        return UpdatePostForm(**kw)
    elif instance is None:
        return CreatePostForm(**kw)
    return None


class CreatePostReportForm(BaseForm, forms.ModelForm):
    reason = forms.ChoiceField(
        label=capfirst(_("reason")),
        choices=PostReport.REASON_CHOICES,
        widget=forms.RadioSelect,
    )

    class Meta:
        model = PostReport
        fields = ("reason", "notes")
        widgets = {"notes": forms.Textarea(attrs={"rows": 4})}

    def __init__(self, *args, **kwargs):
        self.post = kwargs.pop("post")
        super().__init__(*args, **kwargs)

    def save(self):
        instance = super().save(commit=False)
        instance.post = self.post
        instance.save()
        return instance


class HandlePostReportForm(BaseForm, forms.ModelForm):
    moderation_status = forms.ChoiceField(
        label=capfirst(_("moderation status")),
        choices=PostReport.MODERATION_STATUS_CHOICES,
        widget=forms.RadioSelect,
    )

    class Meta:
        model = PostReport
        fields = ("moderation_status",)

    def save(self):
        instance = super().save(commit=False)
        instance.handled_at = timezone.now()
        instance.handled_by = self.request.user
        instance.save()
        instance.post.moderation_status = instance.moderation_status
        instance.post.save()
        signals.post_report_handled.send(
            sender=PostReport, instance=instance, form=self, request=self.request
        )
        return instance



from django.contrib import messages
from django.http import JsonResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.translation import ugettext as _

from tinyforum.forms import (
    form_for_thread,
    form_for_post,
    CreatePostReportForm,
    HandlePostReportForm,
)
from tinyforum.models import Thread, Post, PostReport
from tinyforum.utils import paginate_list, render_detail, render_list


def thread_list(request):
    queryset = Thread.objects.select_related(
        "authored_by__profile", "latest_post__authored_by__profile"
    )

    if request.GET.get("status") == "closed":
        queryset = queryset.closed()
    else:
        queryset = queryset.active()

    return render_list(request, queryset, paginate_by=50)


def post_list(request, pk):
    thread = get_object_or_404(Thread.objects.visible(), pk=pk)
    if request.user.is_authenticated and request.method == "POST":
        form = form_for_post(request, thread=thread)
        if form.is_valid():
            form.save()
            return redirect(thread.get_absolute_url() + "?page=last")
    else:
        form = None

    posts = paginate_list(
        request,
        thread.posts.visible().select_related("authored_by__profile"),
        paginate_by=20,
        orphans=5,
    )

    if (
        request.user.is_authenticated
        and form is None
        and posts.paginator.num_pages == posts.number
    ):
        form = form_for_post(request, thread=thread)

    return render(
        request,
        "tinyforum/post_list.html",
        {"object_list": posts, "post_list": posts, "thread": thread, "form": form},
    )


def thread_form(request, *, pk=None, moderation=False):
    instance = pk and get_object_or_404(Thread, pk=pk)
    form = form_for_thread(request, instance=instance, moderation=moderation)

    if form is None:
        messages.error(request, _("Sorry, you do not have permissions."))
        return redirect(instance)

    if request.method == "POST" and form.is_valid():
        instance = form.save()
        return redirect(instance)

    return render_detail(
        request,
        instance or Thread,
        {
            "form": form,
            "moderation": (
                moderation and instance and instance.authored_by != request.user
            ),
        },
        template_name_suffix="_form",
    )


def thread_star(request, pk):
    instance = get_object_or_404(Thread, pk=pk)
    status = bool(int(request.GET.get("status")))

    if status:
        instance.starred_by.add(request.user)
    else:
        instance.starred_by.remove(request.user)

    return JsonResponse({"thread": instance.pk, "status": int(status)})


def post_form(request, *, pk, moderation=False):
    instance = get_object_or_404(Post.objects.select_related("thread"), pk=pk)
    form = form_for_post(
        request, thread=instance.thread, instance=instance, moderation=moderation
    )

    if form is None:
        messages.error(request, _("Sorry, you do not have permissions."))
        return redirect(instance.thread)

    if request.method == "POST" and form.is_valid():
        form.save()
        return redirect(instance.thread)

    return render_detail(
        request,
        instance,
        {
            "form": form,
            "moderation": (moderation and instance.authored_by != request.user),
        },
        template_name_suffix="_form",
    )


def post_report(request, *, pk):
    instance = get_object_or_404(Post, pk=pk)
    if instance.reports.filter(authored_by=request.user).exists():
        messages.info(request, _("You already reported this post."))
        return redirect(instance.thread)

    kw = {"request": request, "post": instance}
    if request.method == "POST":
        form = CreatePostReportForm(request.POST, **kw)
        if form.is_valid():
            form.save()
            messages.success(
                request,
                _(
                    "Thank you for the report. A community moderator will"
                    " deal with it as soon as possible!"
                ),
            )
            return redirect(instance.thread)
    else:
        form = CreatePostReportForm(**kw)

    return render(
        request, "tinyforum/report_form.html", {"post": instance, "form": form}
    )


def report_list(request):
    return render_list(
        request,
        PostReport.objects.filter(handled_at__isnull=True)
        .exclude(authored_by=request.user)
        .order_by("created_at")
        .select_related("post__authored_by__profile", "authored_by__profile"),
    )


def report_handle(request, *, pk):
    instance = get_object_or_404(
        PostReport.objects.filter(handled_at__isnull=True).exclude(
            authored_by=request.user
        ),
        pk=pk,
    )
    kw = {"request": request, "instance": instance}
    if request.method == "POST":
        form = HandlePostReportForm(request.POST, **kw)
        if form.is_valid():
            form.save()
            return redirect("tinyforum:report-list")
    else:
        form = HandlePostReportForm(**kw)
    return render(
        request, "tinyforum/report_form.html", {"post": instance, "form": form}
    )



from django.contrib import admin

from tinyforum import models


@admin.register(models.Thread)
class ThreadAdmin(admin.ModelAdmin):
    date_hierarchy = "created_at"
    list_display = (
        "title",
        "is_pinned",
        "authored_by",
        "created_at",
        "closed_at",
        "moderation_status",
    )
    list_filter = ("is_pinned", "moderation_status")
    radio_fields = {"moderation_status": admin.HORIZONTAL}
    # starred_by/m2m is unusable with raw_id_fields, but it prevents loading
    # all users into a single list in the admin form.
    raw_id_fields = ("authored_by", "latest_post", "starred_by")
    search_fields = ("title",)


@admin.register(models.Post)
class PostAdmin(admin.ModelAdmin):
    date_hierarchy = "created_at"
    list_display = (
        "__str__",
        "authored_by",
        "thread",
        "created_at",
        "moderation_status",
    )
    list_filter = ("moderation_status",)
    ordering = ["-created_at"]
    radio_fields = {"moderation_status": admin.HORIZONTAL}
    raw_id_fields = ("thread", "authored_by")
    search_fields = ("thread__title", "text")


@admin.register(models.PostReport)
class PostReportAdmin(admin.ModelAdmin):
    list_display = (
        "post",
        "reason",
        "notes",
        "handled_at",
        "handled_by",
        "moderation_status",
    )
    list_filter = ("reason", "handled_at")
    raw_id_fields = ("authored_by", "handled_by", "post")



from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.html import strip_tags
from django.utils.text import Truncator
from django.utils.translation import ugettext_lazy as _

from ckeditor.fields import RichTextField
from html_sanitizer.django import get_sanitizer


class BaseQuerySet(models.QuerySet):
    def visible(self):
        return self.exclude(moderation_status=self.model.HIDDEN)


class BaseModel(models.Model):
    GOOD = "good"
    FLAGGED = "flagged"
    COLLAPSED = "collapsed"
    HIDDEN = "hidden"

    MODERATION_STATUS_CHOICES = (
        (GOOD, _("good")),
        (FLAGGED, _("flagged")),
        (COLLAPSED, _("collapsed")),
        (HIDDEN, _("hidden")),
    )

    created_at = models.DateTimeField(_("created at"), default=timezone.now)
    authored_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        verbose_name=_("authored by"),
    )
    moderation_status = models.CharField(
        _("moderation status"),
        max_length=10,
        choices=MODERATION_STATUS_CHOICES,
        default=GOOD,
    )

    objects = BaseQuerySet.as_manager()

    class Meta:
        abstract = True


class ThreadQuerySet(BaseQuerySet):
    def active(self):
        return self.visible().filter(closed_at__isnull=True)

    def closed(self):
        return self.visible().filter(closed_at__isnull=False)


class Thread(BaseModel):
    title = models.CharField(_("title"), max_length=200)
    is_pinned = models.BooleanField(
        _("is pinned"),
        default=False,
        help_text=_("Pinned threads are shown at the top of the thread list."),
    )
    closed_at = models.DateTimeField(_("closed at"), blank=True, null=True)

    latest_post = models.OneToOneField(
        "Post",
        on_delete=models.SET_NULL,
        related_name="+",
        blank=True,
        null=True,
        verbose_name=_("latest post"),
    )
    post_count = models.IntegerField(_("post count"), default=0)
    starred_by = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        blank=True,
        related_name="starred_threads",
        verbose_name=_("starred by"),
    )

    objects = ThreadQuerySet.as_manager()

    class Meta:
        ordering = ["-is_pinned", "-latest_post__created_at", "-created_at"]
        verbose_name = _("thread")
        verbose_name_plural = _("threads")

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        if self.moderation_status == self.HIDDEN:
            return reverse("tinyforum:thread-list")
        return reverse("tinyforum:thread-detail", kwargs={"pk": self.pk})

    def save(self, *args, **kwargs):
        if self.pk:
            self.post_count = self.posts.visible().count()
            self.latest_post = self.posts.visible().last()
        super().save(*args, **kwargs)

    save.alters_data = True


class Post(BaseModel):
    thread = models.ForeignKey(
        Thread, on_delete=models.CASCADE, verbose_name=_("thread"), related_name="posts"
    )
    text = RichTextField(
        _("text"),
        config_name="tinyforum-post",
        extra_plugins=["emojione"],
        external_plugin_resources=[
            ("emojione", "/static/webapp/lib/ckeditor-emojione-1.0.1/", "plugin.js")
        ],
    )

    class Meta:
        ordering = ["created_at"]
        verbose_name = _("post")
        verbose_name_plural = _("post")

    def __str__(self):
        return Truncator(strip_tags(self.text)).words(20, truncate="...")

    def save(self, *args, **kwargs):
        self.text = get_sanitizer("tinyforum-post").sanitize(self.text)
        super().save(*args, **kwargs)
        self.thread.save()
        # TODO This does not belong into the tinyforum module
        try:
            self.authored_by.profile.post_count = (
                self.authored_by.post_set.visible().count()
            )
            self.authored_by.profile.save()
        except models.ObjectDoesNotExist:  # Profile may not exist here.
            pass

    save.alters_data = True


class Report(BaseModel):
    REASON_CHOICES = (
        ("annoying", _("It's annoying or not interesting")),
        ("misplaced", _("I think it shouldn't be here")),
        ("spam", _("It's spam")),
    )

    reason = models.CharField(_("reason"), max_length=10, choices=REASON_CHOICES)
    notes = models.TextField(
        _("notes"), blank=True, help_text=_("Anything else you want to say?")
    )

    handled_at = models.DateTimeField(_("handled at"), blank=True, null=True)
    handled_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        verbose_name=_("handled by"),
        related_name="+",
    )

    # Override BaseModel.moderation_status with a blank=True version
    moderation_status = models.CharField(
        _("moderation status"),
        max_length=10,
        choices=BaseModel.MODERATION_STATUS_CHOICES,
        blank=True,
    )

    class Meta:
        abstract = True


class PostReport(Report):
    post = models.ForeignKey(
        Post, on_delete=models.CASCADE, related_name="reports", verbose_name=_("post")
    )

    class Meta:
        unique_together = (("authored_by", "post"),)
        verbose_name = _("post report")
        verbose_name_plural = _("post reports")



# Copied from feincms3/shortcuts.py

from django.core import paginator
from django.shortcuts import render


__all__ = ("template_name", "render_list", "render_detail")


def template_name(model, template_name_suffix):
    return "%s/%s%s.html" % (
        model._meta.app_label,
        model._meta.model_name,
        template_name_suffix,
    )


def paginate_list(request, iterable, paginate_by=None, orphans=0):
    if not paginate_by:
        return iterable

    p = paginator.Paginator(iterable, paginate_by)
    page = request.GET.get("page")
    if page == "last":
        return p.page(p.num_pages)

    try:
        return p.page(request.GET.get("page"))
    except paginator.PageNotAnInteger:
        return p.page(1)
    except paginator.EmptyPage:
        return p.page(p.num_pages)


def render_list(
    request,
    queryset,
    context=None,
    *,
    template_name_suffix="_list",
    paginate_by=None,
    orphans=0
):
    context = context or {}
    if paginate_by:
        p = paginator.Paginator(queryset, paginate_by)
        try:
            object_list = p.page(request.GET.get("page"))
        except paginator.PageNotAnInteger:
            object_list = p.page(1)
        except paginator.EmptyPage:
            object_list = p.page(p.num_pages)
    else:
        object_list = queryset

    context.update(
        {
            "object_list": object_list,
            "%s_list" % queryset.model._meta.model_name: object_list,
        }
    )
    return render(request, template_name(queryset.model, template_name_suffix), context)


def render_detail(request, object, context=None, *, template_name_suffix="_detail"):
    context = context or {}
    context.update({"object": object, object._meta.model_name: object})
    return render(request, template_name(object, template_name_suffix), context)



from urllib.parse import urlencode

from django import template
from django.template.loader import render_to_string


register = template.Library()


@register.simple_tag(takes_context=True)
def thread_star(context, thread):
    user = context["request"].user
    if not user.is_authenticated:
        return ""

    if not hasattr(user, "starred_threads_cache"):
        user.starred_threads_cache = list(
            user.starred_threads.values_list("id", flat=True)
        )
    return render_to_string(
        "tinyforum/thread_star.html",
        {"thread": thread, "status": thread.id in user.starred_threads_cache},
    )


@register.simple_tag(takes_context=True)
def pagination_link(context, page):
    query = urlencode(sorted(dict(context["request"].GET.items(), page=page).items()))
    return "?%s" % query if query else "."



#!/usr/bin/env python

import os
from setuptools import setup, find_packages


def read(filename):
    return open(os.path.join(os.path.dirname(__file__), filename)).read()


setup(
    name="django-tinyforum",
    version=__import__("tinyforum").__version__,
    description="Tiny forum for Django",
    long_description=read("README.rst"),
    author="Matthias Kestenholz",
    author_email="mk@feinheit.ch",
    url="http://github.com/matthiask/django-tinyforum/",
    license="MIT License",
    platforms=["OS Independent"],
    packages=find_packages(exclude=["tests", "testapp"]),
    include_package_data=True,
    install_requires=["django-ckeditor", "html-sanitizer"],
    classifiers=[
        # "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    zip_safe=False,
)
